// XXMLFile.h: interface for the CXXMLFile class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_XXMLFILE_H__F5E3CD25_0B84_4191_A1A7_B3669180DFFA__INCLUDED_)
#define AFX_XXMLFILE_H__F5E3CD25_0B84_4191_A1A7_B3669180DFFA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afxtempl.h>


//************************************************************************//
/*!  \class CXXMLFile
 *   \author Manuel Lucas Vias Livschitz (MLVL)
 *   \date 04/05/2002
 *   \brief A class to work with XML files as tree collections.
 */
class CXXMLFile  {
public:
	CXXMLFile();
	virtual ~CXXMLFile();
public:
	// Abstract node
	class CElementPart;
	class CElementPart{
	public:
		CElementPart* m_Parent;
		CString m_Text;
		enum TType { TElement, TText, TComment } m_Type;
	public:
		CElementPart(){};
		virtual ~CElementPart(){};
	};

	// Element node
	class CElement : public CElementPart, public CList<CElementPart*,CElementPart*>{
	public:
		CElement(){ m_Type=TElement; };
		virtual ~CElement();
	public:
		CMapStringToString AttributeToValue;
	};

	// Text node
	class CText : public CElementPart{
	public:
		CText(){ m_Type=TText; };
	};

	// Comment node
	class CComment : public CElementPart{
	public:
		CComment(){ m_Type=TComment; };
	};
public:
	// The error list, after executing Write() or Read() (if any)
	CStringList m_ErrorList;

	// Symbol table functions
	void RemoveAll();
	void AddSymbol(CString coded, CString decoded);
	void DefaultSymbols();
	void ClearSymbols();
	void AddOpenTag(CString tag);

	// File manipulation routines
	bool Write();
	CString GetFile();
	void SetFile(CString filename);
	bool Read();

	// Tree manipulation routines
	CElementPart* Root();
	CElementPart* AddElement(CElementPart* Parent);
	CElementPart* AddComment(CElementPart* Parent, CString text);
	CElementPart* AddText(CElementPart* Parent, CString text);
	void SetText(CElementPart* node, CString text);
	void GetText(CElementPart* node, CString &text);
	bool IsElement(CElementPart* node);
	bool IsComment(CElementPart* node);
	bool IsText(CElementPart* node);
	CMapStringToString* GetElementAttrMap(CElementPart* node);
	bool BuildChildList(CElementPart* node, CList<CElementPart*,CElementPart*> &l);
private:
	// The XML tree
	CElementPart* m_Root;

	// Symbol codification functions
	void Decodify(CString &html);
	void Codify(CString &html);
	CMapStringToString m_Symbols; // encoded to decoded

	// Part management
	void WritePart(CStdioFile *f, CElementPart * p, int Depth, bool bNoIdent=false);

	// Filename
	CString m_Filename;

	// Open tags map (acts like a set, so second value is ignored)
	CMapStringToString m_OpenTags;
};

#endif // !defined(AFX_XXMLFILE_H__F5E3CD25_0B84_4191_A1A7_B3669180DFFA__INCLUDED_)
